
// source: https://gabrieleserra.ml/blog/2020-08-27-an-introduction-to-gcc-and-gccs-plugins.html
// GCC only
// Use  gcc -g -finstrument-functions -no-pie
// addr2line -paf -e a.out ADDR

#include <stdio.h>

// Note:
// We need 'no_instrument_function' attribute on __cyg_profile_fun_*
#define NO_INSTRUMENT __attribute__ ((no_instrument_function))

// called every time a function get called
NO_INSTRUMENT void __cyg_profile_func_enter (void *this_fn, void *call_site) {
    printf("Entering in 0x%lx from 0x%lx...\n", (unsigned long)this_fn, (unsigned long)call_site);
}

// called every time a function returns
NO_INSTRUMENT void __cyg_profile_func_exit  (void *this_fn, void *call_site) {
    printf("Exiting from 0x%lx to 0x%lx...\n", (unsigned long)this_fn, (unsigned long)call_site);
}

// function
void f2() {
  printf("f2()\n");
}

// function
void f1() {
  printf("f1() begin\n");
  f2();
  printf("f1() end\n");
}

int main() {
  printf("main() begin\n");
  f1();
  printf("main() end\n");
}

