
// source: Florian Merz
// strict aliasing rules + overlap

#include <stdio.h>
#include <string.h>             // memcpy, memmove

struct S {
    int a;
    int b;
    int c;
};                              // memory: [a,b,c]

void printS(struct S s)
{
    printf("{%d,%d,%d}\n", s.a, s.b, s.c);
}

struct T {
    int p;
    struct S s;
};                              // memory: [p,a,b,c]

// union U: overlapping S and T
union U {
    struct S s;
    struct T t;
} u;

void copy(struct S *p, struct S *q)
{
    *p = *q;                    // problem = overlap
}

int main(int c, char **v)
{
    u.s.b = 1;
    printf("sizeS=%zd, sizeT=%zd, sizeU=%zd\n",
           sizeof(u.s), sizeof(u.t), sizeof(u));
    printf("u.s:   ");
    printS(u.s);
#if defined(CORRECT)
    // no problem with overlaping memory areas
    memmove(&u.t.s, &u.s, sizeof(u.t.s));       // O.K.
#else
    // all 3 variants have aliasing problem
    copy(&u.t.s, &u.s);                         // = problem
    //u.t.s=u.s;                                  // = problem
    //memcpy(&u.t.s, &u.s, sizeof(u.t.s));        // = problem
#endif
    printf("u.t.s = u.s; // overlapping structures\n");
    printf("u.t.s: ");
    printS(u.t.s);
    return 0;
}
