
// priklad ze zkousky

#include <assert.h>
#include <stdbool.h>
#include <ctype.h>
#include <stdio.h>
#include <limits.h>
#include <string.h>

// pomocna funkce
// obecny prevod znaku na hodnotu cislice (az do zakladu 36)
// predpokladame ASCII (nebo UTF-8), nefunguje pro EBCDIC
int char2digit(int c)
{
    if (isdigit(c))
        return c - '0';
    c = toupper(c);
    if (isalpha(c))
        return c - 'A' + 10;
    return 99;          // neni cislice v zadne soustave
}

// prevod  retezec->cislo
unsigned long str2ul(const char *s, const char **p, int radix)
{
    assert(radix >= 2 && radix <= 36);

    while (isspace(*s))
        s++;                    // preskocime pripadne bile znaky

    unsigned long result = 0;
    int digit;
    while ((digit = char2digit(*s)) < radix) {
        // je dalsi platna cislice
        // kontrola preteceni
        if (result > (ULONG_MAX-digit)/radix) break;
        result = result * radix + digit;
        s++;
    }
    // konec prevodu - neni dalsi cislice NEBO by přeteklo
    if (p)
        *p = s;                 // vratime pozici, pokud je pozadovano (!=NULL)

    return result;              // vraceni vysledku
}


// prevod  cislo->retezec
bool ul2str(unsigned long x, int radix, char *s, int n)
{
    assert(radix >= 2 && radix <= 36);

    char *p = s;
    do {
        int digit = x % radix;  // koncova cislice
        x /= radix;             // odstraneni koncove cislice

        if (--n <= 0)           // kontrola limitu
            return false;

        // prevod cislice na znak v retezci
        *s++ = (digit < 10) ? '0' + digit : 'A' + digit - 10;

    } while (x != 0);           // do-while zvladne dobre x==0

    // ukoncit C-string
    *s-- = '\0';

    // otocit poradi -- prevod ukladal obracene
    while (p < s) {
        char tmp = *p;
        *p = *s;
        *s = tmp;
        p++; s--;
    }

    return true;                // hotovo
}

#define TEST(p)  do{ printf("%s\n",#p); p; }while(0)
#define N 80

int main(void)
{
    unsigned long x = 12345;
    char s[N] = "";
    bool b;
    printf("\n");
    int RADIX = 2;
    printf("sizeof(s) = %zd\n", sizeof(s));
    printf("\n");
#if 1
    TEST(b = ul2str(x, RADIX=2, s, sizeof(s)));
    if (b)
        printf("uspesny prevod: %lu -> \"%s\"\n", x, s);
    else
        printf("NEuspesny prevod: %lu -> \"%s\"\n", x, s);
    printf("\n");
#endif
    const char *p;
    TEST(x = str2ul(s, &p, RADIX=2));
    printf("prevod: \"%s\" -> %lu\n", s, x);
    printf("prevod skoncil na pozici: %d v \"%s\" strlen(s)=%zd\n",
            (int)(p - s), s, strlen(s));
    printf("\n");

    printf("test preteceni\n");
    strncpy(s,"1234567890123456789012345678901234567890",sizeof(s)-1);
    s[sizeof(s)-1]='\0';
    TEST(x = str2ul(s, &p, RADIX=10));
    printf("prevod: \"%s\" -> %lu\n", s, x);
    printf("prevod skoncil na pozici: %d ", (int)(p - s));
    printf(" \"%.*s'%s\"\n", (int)(p - s), s, s+(p-s));
    printf("\n");

    printf("test chybneho vstupu\n");
    strncpy(s,"123xxx321",sizeof(s)-1);
    s[sizeof(s)-1]='\0';
    TEST(x = str2ul(s, &p, RADIX=10));
    printf("prevod: \"%s\" -> %lu\n", s, x);
    printf("prevod skoncil na pozici: %d ", (int)(p - s));
    printf(" \"%.*s'%s\"\n", (int)(p - s), s, s+(p-s));
    printf("\n");
}

