
// How to use asynchronous signals correctly
// Source: SEI CERT C Coding Standard

#include <signal.h>
#include <stdlib.h>
#include <string.h>
#include <stdio.h>

enum { MAX_MSG_SIZE = 24 };

volatile sig_atomic_t e_flag = 0;

void handler(int signum) {
    e_flag = 1;         // this is safe
    // can NOT use printf and many other functions here
}

int main(void) {
    char err_msg[MAX_MSG_SIZE] = { "No errors." };
    signal(SIGINT, handler);
    printf("Press Ctrl-C to break\n");

    int end = 0;
    while (!e_flag && !end) {
        // do work
    }

    if (e_flag) {
        strcpy(err_msg, "SIGINT received.");
        // do error handling
    }
    printf("\n%s\n",err_msg);
    // do clean up
}
