
// GSL - GNU Scientific Library example
// Integrate:
//   \int_0^1 x^{-1/2} log(x) dx = -4
// Source: GSL manual
// gcc gsl-quadrature-test.c -lm -lgsl 
// TODO: needs more comments

#include <stdio.h>
#include <math.h>

#include <gsl/gsl_integration.h>

// function to integrate
double f (double x, void * params) {
  double alpha = *(double *) params;    // will be 1.0
  double f = log(alpha*x) / sqrt(x);
  return f;
}

#define SIZE 100        // max number of intervals?

int main(void)
{
  // temporary ?
  gsl_integration_workspace * w = gsl_integration_workspace_alloc(SIZE);

  double alpha = 1.0;
  gsl_function F;
  F.function = &f;
  F.params = &alpha;    // closure data ?

  // numerical quadrature
  double result, error;
  gsl_integration_qags (&F, 0, 1, 0, 1e-7, SIZE, w, &result, &error);

  printf ("result          = % .16f\n", result);
  printf ("estimated error = % .16f\n", error);
  double expected = -4.0;
  printf ("exact result    = % .16f\n", expected);
  printf ("actual error    = % .16f\n", result - expected);
  printf ("intervals       = %zu\n", w->size);

  gsl_integration_workspace_free (w);

  return 0;
}

