/* main-test.c
 * Main code for loop unroll test
 */

extern void f(long p[], long n);

#define N 1000L
long pole[N];

/**************************************************************************
 *  function for reading 64 bit timer (Pentium, K6, etc.) 
 *  for GNU C/C++ only 
 */
#ifndef __GNUC__
#error "use GNU C, please"
#endif

// universal x86, x86-64 code:
static __inline__ unsigned long long rdtsc(void)
{
    unsigned a, d;
    __asm volatile ("rdtsc":"=a" (a), "=d"(d)); // compiler-dependent
    return ((unsigned long long) a) | (((unsigned long long) d) << 32);
}

/**************************************************************************/

#include <stdio.h>
#include <assert.h>

#define M 11

void measure(void (*f) (long p[], long n))
{
    int i;
    // measure M times, use the median
    long long a[M];
    for (i = 0; i < M; i++) {
        long long t0 = rdtsc();
        f(pole, N);
        long long t = rdtsc() - t0;
        a[i] = t;
    }
    for(i = 0; i <= M/2; i++)  // partial sort
        for(int j=i+1; j<M; j++)
            if (a[i]>a[j]) { 
                // swap
                long long tmp=a[i];
                a[i] = a[j];
                a[j] = tmp;
            }
    // for (i = 0; i < M; i++) printf("a[%d]=%Ld ", i, a[i]); // DEBUG
    long long median = a[M/2];
    printf("time=%Ld\n", median);
}

int main(int argc, char *argv[])
{
    printf("%s\n", argv[0]);
    assert(N % 10 == 0);
    measure(f);
    measure(f);
    measure(f);
    return 0;
}
