
// Example: C pointers
// Author: P.Peringer, FIT BUT Brno, (public domain)
// Keywords: pointer, pointer to pointer, pointer size, byteorder
// Use gcc -std=c99 or c11

#include <stdio.h>
#include <limits.h>     // CHAR_BIT

int a = 5;
int *pa = &a;           // pointer to int variable a
int **ppa = &pa;        // pointer to pointer to int
// Warning: using more than 2 levels of indirection is unusual
int ***pppa = &ppa;     // pointer to pointer to pointer to int

int main(void)
{
    // struct should be local due to non-constant initializer
    struct {
        void *ptr;              // pointer value = address
        size_t tgt_size;        // size of pointer target
    } info[] = {
        { pa, sizeof(*pa) },
        { ppa, sizeof(*ppa) },
        { pppa, sizeof(*pppa) },
    };

    printf("Address space size = 2^{%zd}\n", sizeof(void*)*CHAR_BIT);
    for (int i = 0; i < (sizeof(info) / sizeof(info[0])); ++i) {
        printf("%p: char[%zd] = { ", info[i].ptr, info[i].tgt_size);
        for (int o = 0; o < info[i].tgt_size; ++o) {
            printf("%02x, ", ((unsigned char *) info[i].ptr)[o]);
        }
        printf("}\n");
    }
    return 0;
}

