
// Example: C pointers vs arrays
// Author: P.Peringer, FIT BUT Brno, (public domain)
// Keywords: pointer, pointer to array item, pointer to array
// Use gcc -std=c99 or c11

#include <stdio.h>
#include <limits.h>     // CHAR_BIT

int a[5] = { 0, 1, 2, 3, }; // array
int *p = a;             // pointer to a[0]
int **pp = &p;          // pointer to pointer to a[0]
int (*pa)[5] = &a;      // pointer to array

int main(void)
{
    // struct should be local due to non-constant initializer
    struct {
        void *ptr;              // pointer value = address
        size_t tgt_size;        // size of pointer target
    } info[] = {
        { a, sizeof(a) },
        { p, sizeof(*p) },
        { pp, sizeof(*pp) },
        { pa, sizeof(*pa) },
    };

    printf("Address space size = 2^{%zd}\n", sizeof(void*)*CHAR_BIT);
    for (int i = 0; i < (sizeof(info) / sizeof(info[0])); ++i) {
        printf("%p: char[%zd] = { ", info[i].ptr, info[i].tgt_size);
        for (int o = 0; o < info[i].tgt_size; ++o) {
            printf("%02x,", ((unsigned char *) info[i].ptr)[o]);
        }
        printf(" }\n");
    }
    return 0;
}

