
// test integer overflow undefined behavior (UB)
// try:
//   gcc -O2            ## overflow is UB, dead code
//   gcc -O2 -fwrapv    ## ignore overflow, wrap
//   gcc -O2 -ftrapv    ## catch overflow, abort

// TODO: C23 stdchkdint.h

/*
citation from GCC manual:

-fstrict-overflow
    Allow the compiler to assume strict signed overflow rules, depending on
    the language being compiled. For C (and C++) this means that overflow when
    doing arithmetic with signed numbers is undefined, which means that the
    compiler may assume that it will not happen. This permits various
    optimizations. For example, the compiler will assume that an expression like
    i + 10 > i will always be true for signed i. This assumption is only valid if
    signed overflow is undefined, as the expression is false if i + 10 overflows
    when using twos complement arithmetic. When this option is in effect any
    attempt to determine whether an operation on signed numbers will overflow must
    be written carefully to not actually involve overflow.

        This option also allows the compiler to assume strict pointer semantics:
    given a pointer to an object, if adding an offset to that pointer does not
    produce a pointer to the same object, the addition is undefined. This permits
    the compiler to conclude that p + u > p is always true for a pointer p and
    unsigned integer u. This assumption is only valid because pointer wraparound
    is undefined, as the expression is false if p + u overflows using twos
    complement arithmetic.

        See also the -fwrapv option. Using -fwrapv means that integer signed
    overflow is fully defined: it wraps. When -fwrapv is used, there is no
    difference between -fstrict-overflow and -fno-strict-overflow for integers.
    With -fwrapv certain types of overflow are permitted. For example, if the
    compiler gets an overflow when doing arithmetic on constants, the overflowed
    value can still be used with -fwrapv, but not otherwise.

    The -fstrict-overflow option is enabled at levels -O2, -O3, -Os.

-ftrapv
    This option generates traps for signed overflow on addition, subtraction,
    multiplication operations.

-fwrapv
    This option instructs the compiler to assume that signed arithmetic
    overflow of addition, subtraction and multiplication wraps around using
    twos-complement representation. This flag enables some optimizations and
    disables others. This option is enabled by default for the Java front-end, as
    required by the Java language specification.

*/

#include <stdio.h>
#include <stdlib.h>     // exit()
#include <limits.h>     // INT_MAX

int main()
{
    int i;
    printf("INT_MAX = %d\n", INT_MAX);
    scanf("%d", &i);
    int save = i;
    printf("input   = %d\n", save);
#ifndef CORRECT
    // incorrect overflow checking:
    if (i + 10 > i) {           // possible UB, always true
        i+=10;                  // possible UB
    }else{
        printf("* overflow detected\n");   // dead code
        exit(1);
    }
#else
    // correct overflow checking:
    if (INT_MAX - 10 >= i) {    // well defined
        i+=10;                  // never overflow
    }else{
        printf("* Overflow detected\n");   // never optimized out
        exit(1);
    }
#endif
    printf("%d + 10 = %d\n", save, i);
}
